@@ -9,7 +9,7 @@ from concurrent.futures import ThreadPoolExecutor
 # logging.basicConfig(level=logging.INFO)
 # logging.getLogger("openai").setLevel(logging.DEBUG)
 
-OPENAI_MODEL = os.environ.get("OPENAI_MODEL", "o3")
+OPENAI_MODEL = os.environ.get("OPENAI_MODEL", "gpt-5")
 
 ENABLE_CODE_SNIPPET_EXCLUSION = True
 # gpt-4.5 needed this for better quality
@@ -26,6 +26,8 @@ search:
 source_dir = "docs"
 languages = {
     "ja": "Japanese",
+    "ko": "Korean",
+    "zh": "Chinese",
     # Add more languages here, e.g., "fr": "French"
 }
 
@@ -79,6 +81,64 @@ eng_to_non_eng_mapping = {
         "Python first": "Python ファースト",
         # Add more Japanese mappings here
     },
+    "ko": {
+        "agents": "에이전트",
+        "computer use": "컴퓨터 사용",
+        "OAI hosted tools": "OpenAI 호스트하는 도구",
+        "well formed data": "적절한 형식의 데이터",
+        "guardrail": "가드레일",
+        "orchestrating multiple agents": "멀티 에이전트 오케스트레이션",
+        "handoffs": "핸드오프",
+        "function tools": "함수 도구",
+        "function calling": "함수 호출",
+        "tracing": "트레이싱",
+        "code examples": "코드 예제",
+        "vector store": "벡터 스토어",
+        "deep research": "딥 리서치",
+        "category": "카테고리",
+        "user": "사용자",
+        "parameter": "매개변수",
+        "processor": "프로세서",
+        "server": "서버",
+        "web search": "웹 검색",
+        "file search": "파일 검색",
+        "streaming": "스트리밍",
+        "system prompt": "시스템 프롬프트",
+        "Python-first": "파이썬 우선",
+        "interruption": "인터럽션(중단 처리)",
+        "TypeScript-first": "TypeScript 우선",
+        "Human in the loop": "휴먼인더루프 (HITL)",
+        "Hosted tool": "호스티드 툴",
+        "Hosted MCP server tools": "호스티드 MCP 서버 도구",
+        "raw": "원문",
+        "Realtime Agents": "실시간 에이전트",
+        "Build your first agent in minutes.": "단 몇 분 만에 첫 에이전트를 만들 수 있습니다",
+        "Let's build": "시작하기",
+    },
+    "zh": {
+        "agents": "智能体",
+        "computer use": "计算机操作",
+        "OAI hosted tools": "由OpenAI托管的工具",
+        "well formed data": "格式良好的数据",
+        "guardrail": "安全防护措施",
+        "handoffs": "任务转移",
+        "function tools": "工具调用",
+        "tracing": "追踪",
+        "code examples": "代码示例",
+        "vector store": "向量存储",
+        "deep research": "深度研究",
+        "category": "目录",
+        "user": "用户",
+        "parameter": "参数",
+        "processor": "进程",
+        "server": "服务",
+        "web search": "网络检索",
+        "file search": "文件检索",
+        "streaming": "流式传输",
+        "system prompt": "系统提示词",
+        "Python first": "Python 优先",
+        # Add more mappings here
+    },
     # Add more languages here
 }
 eng_to_non_eng_instructions = {
@@ -87,6 +147,7 @@ eng_to_non_eng_instructions = {
         "* The term 'primitives' can be translated as basic components.",
         "* When the terms 'instructions' and 'tools' are mentioned as API parameter names, they must be kept as is.",
         "* The terms 'temperature', 'top_p', 'max_tokens', 'presence_penalty', 'frequency_penalty' as parameter names must be kept as is.",
+        "* Keep the original structure like `* **The thing**: foo`; this needs to be translated as `* **(translation)**: (translation)`",
     ],
     "ja": [
         "* The term 'result' in the Runner guide context must be translated like 'execution results'",
@@ -94,6 +155,19 @@ eng_to_non_eng_instructions = {
         "* You must consistently use polite wording such as です/ます rather than である/なのだ.",
         # Add more Japanese mappings here
     ],
+    "ko": [
+        "* 공손하고 중립적인 문체(합니다/입니다체)를 일관되게 사용하세요.",
+        "* 개발자 문서이므로 자연스러운 의역을 허용하되 정확성을 유지하세요.",
+        "* 'instructions', 'tools' 같은 API 매개변수와 temperature, top_p, max_tokens, presence_penalty, frequency_penalty 등은 영문 그대로 유지하세요.",
+        "* 문장이 아닌 불릿 항목 끝에는 마침표를 찍지 마세요.",
+    ],
+    "zh": [
+        "* The term 'examples' must be code examples when the page mentions the code examples in the repo, it can be translated as either 'code examples' or 'sample code'.",
+        "* The term 'primitives' can be translated as basic components.",
+        "* When the terms 'instructions' and 'tools' are mentioned as API parameter names, they must be kept as is.",
+        "* The terms 'temperature', 'top_p', 'max_tokens', 'presence_penalty', 'frequency_penalty' as parameter names must be kept as is.",
+        "* Keep the original structure like `* **The thing**: foo`; this needs to be translated as `* **(translation)**: (translation)`",
+    ],
     # Add more languages here
 }
 
@@ -135,12 +209,36 @@ You must return **only** the translated markdown. Do not include any commentary,
   - Fenced code blocks delimited by ``` or ~~~, including all comments inside them.
   - Link URLs inside `[label](URL)` – translate the label, never the URL.
 
+#########################
+##  HARD CONSTRAINTS   ##
+#########################
+- Never insert spaces immediately inside emphasis markers. Use `**bold**`, not `** bold **`.
+- Preserve the number of emphasis markers from the source: if the source uses `**` or `__`, keep the same pair count.
+- Ensure one space after heading markers: `##Heading` -> `## Heading`.
+- Ensure one space after list markers: `-Item` -> `- Item`, `*Item` -> `* Item` (does not apply to `**`).
+- Trim spaces inside link/image labels: `[ Label ](url)` -> `[Label](url)`.
+
+###########################
+##  GOOD / BAD EXAMPLES  ##
+###########################
+- Good: This is **bold** text.
+- Bad:  This is ** bold ** text.
+- Good: ## Heading
+- Bad:  ##Heading
+- Good: - Item
+- Bad:  -Item
+- Good: [Label](https://example.com)
+- Bad:  [ Label ](https://example.com)
+
 #########################
 ##  LANGUAGE‑SPECIFIC  ##
 #########################
 *(applies only when {target_language} = Japanese)*  
 - Insert a half‑width space before and after all alphanumeric terms.  
 - Add a half‑width space just outside markdown emphasis markers: ` **太字** ` (good) vs `** 太字 **` (bad).
+*(applies only when {target_language} = Korean)*  
+- Do not alter spaces around code/identifiers; keep them as in the original.  
+- Do not add stray spaces around markdown emphasis: `**굵게**` (good) vs `** 굵게 **` (bad).
 
 #########################
 ##  DO NOT TRANSLATE   ##
@@ -158,6 +256,7 @@ Translate these terms exactly as provided (no extra spaces):
 ##  EXTRA GUIDELINES   ##
 #########################
 {specific_instructions}
+- When translating Markdown tables, preserve the exact table structure, including all delimiters (|), header separators (---), and row/column counts. Only translate the cell contents. Do not add, remove, or reorder columns or rows.
 
 #########################
 ##  IF UNSURE          ##
@@ -172,7 +271,11 @@ Follow the following workflow to translate the given markdown text data:
 
 1. Read the input markdown text given by the user.
 2. Translate the markdown file into {target_language}, carefully following the requirements above.
-3. Perform a self-review to evaluate the quality of the translation, focusing on naturalness, accuracy, and consistency in detail.
+3. Perform a self-review to check for the following common issues:
+   - Naturalness, accuracy, and consistency throughout the text.
+   - Spacing inside markdown syntax such as `*` or `_`; `**bold**` is correct whereas `** bold **` is not.
+   - Unwanted spaces inside link or image labels, such as `[ Label ](url)`.
+   - Headings or list markers missing a space after their marker.
 4. If improvements are necessary, refine the content without changing the original meaning.
 5. Continue improving the translation until you are fully satisfied with the result.
 6. Once the final output is ready, return **only** the translated markdown text. No extra commentary.
@@ -207,7 +310,7 @@ def translate_file(file_path: str, target_path: str, lang_code: str) -> None:
             code_block_chunks.append(line)
             if in_code_block is True:
                 code_blocks.append("\n".join(code_block_chunks))
-                current_chunk.append(f"CODE_BLOCK_{(len(code_blocks) - 1):02}")
+                current_chunk.append(f"CODE_BLOCK_{(len(code_blocks) - 1):03}")
                 code_block_chunks.clear()
             in_code_block = not in_code_block
             continue
@@ -222,7 +325,16 @@ def translate_file(file_path: str, target_path: str, lang_code: str) -> None:
     translated_content: list[str] = []
     for chunk in chunks:
         instructions = built_instructions(languages[lang_code], lang_code)
-        if OPENAI_MODEL.startswith("o"):
+        if OPENAI_MODEL.startswith("gpt-5"):
+            response = openai_client.responses.create(
+                model=OPENAI_MODEL,
+                instructions=instructions,
+                input=chunk,
+                reasoning={"effort": "low"},
+                text={"verbosity": "low"},
+            )
+            translated_content.append(response.output_text)
+        elif OPENAI_MODEL.startswith("o"):
             response = openai_client.responses.create(
                 model=OPENAI_MODEL,
                 instructions=instructions,
@@ -240,7 +352,7 @@ def translate_file(file_path: str, target_path: str, lang_code: str) -> None:
 
     translated_text = "\n".join(translated_content)
     for idx, code_block in enumerate(code_blocks):
-        translated_text = translated_text.replace(f"CODE_BLOCK_{idx:02}", code_block)
+        translated_text = translated_text.replace(f"CODE_BLOCK_{idx:03}", code_block)
 
     # FIXME: enable mkdocs search plugin to seamlessly work with i18n plugin
     translated_text = SEARCH_EXCLUSION + translated_text
