@@ -2,7 +2,9 @@ from __future__ import annotations
 
 import asyncio
 
-from agents import Agent, Runner, function_tool, set_tracing_disabled
+from pydantic import BaseModel
+
+from agents import Agent, ModelSettings, Runner, function_tool, set_tracing_disabled
 
 """This example uses the built-in support for LiteLLM. To use this, ensure you have the
 ANTHROPIC_API_KEY environment variable set.
@@ -10,6 +12,9 @@ ANTHROPIC_API_KEY environment variable set.
 
 set_tracing_disabled(disabled=True)
 
+# import logging
+# logging.basicConfig(level=logging.DEBUG)
+
 
 @function_tool
 def get_weather(city: str):
@@ -17,13 +22,20 @@ def get_weather(city: str):
     return f"The weather in {city} is sunny."
 
 
+class Result(BaseModel):
+    output_text: str
+    tool_results: list[str]
+
+
 async def main():
     agent = Agent(
         name="Assistant",
         instructions="You only respond in haikus.",
         # We prefix with litellm/ to tell the Runner to use the LitellmModel
-        model="litellm/anthropic/claude-3-5-sonnet-20240620",
+        model="litellm/anthropic/claude-sonnet-4-5-20250929",
         tools=[get_weather],
+        model_settings=ModelSettings(tool_choice="required"),
+        output_type=Result,
     )
 
     result = await Runner.run(agent, "What's the weather in Tokyo?")
