@@ -1,12 +1,12 @@
 """
 Example demonstrating how to use the reasoning content feature with models that support it.
 
-Some models, like deepseek-reasoner, provide a reasoning_content field in addition to the regular content.
+Some models, like gpt-5, provide a reasoning_content field in addition to the regular content.
 This example shows how to access and use this reasoning content from both streaming and non-streaming responses.
 
 To run this example, you need to:
 1. Set your OPENAI_API_KEY environment variable
-2. Use a model that supports reasoning content (e.g., deepseek-reasoner)
+2. Use a model that supports reasoning content (e.g., gpt-5)
 """
 
 import asyncio
@@ -14,12 +14,13 @@ import os
 from typing import Any, cast
 
 from openai.types.responses import ResponseOutputRefusal, ResponseOutputText
+from openai.types.shared.reasoning import Reasoning
 
 from agents import ModelSettings
 from agents.models.interface import ModelTracing
 from agents.models.openai_provider import OpenAIProvider
 
-MODEL_NAME = os.getenv("EXAMPLE_MODEL_NAME") or "deepseek-reasoner"
+MODEL_NAME = os.getenv("EXAMPLE_MODEL_NAME") or "gpt-5"
 
 
 async def stream_with_reasoning_content():
@@ -36,30 +37,30 @@ async def stream_with_reasoning_content():
     reasoning_content = ""
     regular_content = ""
 
+    output_text_already_started = False
     async for event in model.stream_response(
         system_instructions="You are a helpful assistant that writes creative content.",
         input="Write a haiku about recursion in programming",
-        model_settings=ModelSettings(),
+        model_settings=ModelSettings(reasoning=Reasoning(effort="medium", summary="detailed")),
         tools=[],
         output_schema=None,
         handoffs=[],
         tracing=ModelTracing.DISABLED,
         previous_response_id=None,
+        conversation_id=None,
         prompt=None,
     ):
         if event.type == "response.reasoning_summary_text.delta":
-            print(
-                f"\033[33m{event.delta}\033[0m", end="", flush=True
-            )  # Yellow for reasoning content
+            # Yellow for reasoning content
+            print(f"\033[33m{event.delta}\033[0m", end="", flush=True)
             reasoning_content += event.delta
         elif event.type == "response.output_text.delta":
-            print(f"\033[32m{event.delta}\033[0m", end="", flush=True)  # Green for regular content
+            if not output_text_already_started:
+                print("\n")
+                output_text_already_started = True
+            # Green for regular content
+            print(f"\033[32m{event.delta}\033[0m", end="", flush=True)
             regular_content += event.delta
-
-    print("\n\nReasoning Content:")
-    print(reasoning_content)
-    print("\nRegular Content:")
-    print(regular_content)
     print("\n")
 
 
@@ -77,12 +78,13 @@ async def get_response_with_reasoning_content():
     response = await model.get_response(
         system_instructions="You are a helpful assistant that explains technical concepts clearly.",
         input="Explain the concept of recursion in programming",
-        model_settings=ModelSettings(),
+        model_settings=ModelSettings(reasoning=Reasoning(effort="medium", summary="detailed")),
         tools=[],
         output_schema=None,
         handoffs=[],
         tracing=ModelTracing.DISABLED,
         previous_response_id=None,
+        conversation_id=None,
         prompt=None,
     )
 
@@ -102,12 +104,10 @@ async def get_response_with_reasoning_content():
                     refusal_item = cast(Any, content_item)
                     regular_content = refusal_item.refusal
 
-    print("\nReasoning Content:")
+    print("\n\n### Reasoning Content:")
     print(reasoning_content or "No reasoning content provided")
-
-    print("\nRegular Content:")
+    print("\n\n### Regular Content:")
     print(regular_content or "No regular content provided")
-
     print("\n")
 
 
@@ -118,7 +118,7 @@ async def main():
     except Exception as e:
         print(f"Error: {e}")
         print("\nNote: This example requires a model that supports reasoning content.")
-        print("You may need to use a specific model like deepseek-reasoner or similar.")
+        print("You may need to use a specific model like gpt-5 or similar.")
 
 
 if __name__ == "__main__":
