@@ -1,6 +1,7 @@
+from openai.types.shared.reasoning import Reasoning
 from pydantic import BaseModel
 
-from agents import Agent
+from agents import Agent, ModelSettings
 
 PROMPT = (
     "You are a helpful research assistant. Given a query, come up with a set of web searches "
@@ -24,6 +25,7 @@ class WebSearchPlan(BaseModel):
 planner_agent = Agent(
     name="PlannerAgent",
     instructions=PROMPT,
-    model="gpt-4o",
+    model="gpt-5",
+    model_settings=ModelSettings(reasoning=Reasoning(effort="medium")),
     output_type=WebSearchPlan,
 )
