@@ -12,7 +12,10 @@ INSTRUCTIONS = (
 
 search_agent = Agent(
     name="Search agent",
+    model="gpt-4.1",
     instructions=INSTRUCTIONS,
     tools=[WebSearchTool()],
+    # Note that gpt-5 model does not support tool_choice="required",
+    # so if you want to migrate to gpt-5, you'll need to use "auto" instead
     model_settings=ModelSettings(tool_choice="required"),
 )
