@@ -1,8 +1,10 @@
-from typing import cast
-from unittest.mock import AsyncMock, Mock, PropertyMock
+import asyncio
+from typing import Any, cast
+from unittest.mock import AsyncMock, Mock, PropertyMock, patch
 
 import pytest
 
+from agents.exceptions import UserError
 from agents.guardrail import GuardrailFunctionOutput, OutputGuardrail
 from agents.handoffs import Handoff
 from agents.realtime.agent import RealtimeAgent
@@ -22,6 +24,7 @@ from agents.realtime.events import (
     RealtimeToolStart,
 )
 from agents.realtime.items import (
+    AssistantAudio,
     AssistantMessageItem,
     AssistantText,
     InputAudio,
@@ -45,12 +48,202 @@ from agents.realtime.model_events import (
     RealtimeModelTurnEndedEvent,
     RealtimeModelTurnStartedEvent,
 )
-from agents.realtime.model_inputs import RealtimeModelSendSessionUpdate
+from agents.realtime.model_inputs import (
+    RealtimeModelSendAudio,
+    RealtimeModelSendInterrupt,
+    RealtimeModelSendSessionUpdate,
+    RealtimeModelSendUserInput,
+)
 from agents.realtime.session import RealtimeSession
 from agents.tool import FunctionTool
 from agents.tool_context import ToolContext
 
 
+class _DummyModel(RealtimeModel):
+    def __init__(self) -> None:
+        super().__init__()
+        self.events: list[Any] = []
+        self.listeners: list[Any] = []
+
+    async def connect(self, options=None):  # pragma: no cover - not used here
+        pass
+
+    async def close(self):  # pragma: no cover - not used here
+        pass
+
+    async def send_event(self, event):
+        self.events.append(event)
+
+    def add_listener(self, listener):
+        self.listeners.append(listener)
+
+    def remove_listener(self, listener):
+        if listener in self.listeners:
+            self.listeners.remove(listener)
+
+
+@pytest.mark.asyncio
+async def test_property_and_send_helpers_and_enter_alias():
+    model = _DummyModel()
+    agent = RealtimeAgent(name="agent")
+    session = RealtimeSession(model, agent, None)
+
+    # property
+    assert session.model is model
+
+    # enter alias calls __aenter__
+    async with await session.enter():
+        # send helpers
+        await session.send_message("hi")
+        await session.send_audio(b"abc", commit=True)
+        await session.interrupt()
+
+        # verify sent events
+        assert any(isinstance(e, RealtimeModelSendUserInput) for e in model.events)
+        assert any(isinstance(e, RealtimeModelSendAudio) and e.commit for e in model.events)
+        assert any(isinstance(e, RealtimeModelSendInterrupt) for e in model.events)
+
+
+@pytest.mark.asyncio
+async def test_aiter_cancel_breaks_loop_gracefully():
+    model = _DummyModel()
+    agent = RealtimeAgent(name="agent")
+    session = RealtimeSession(model, agent, None)
+
+    async def consume():
+        async for _ in session:
+            pass
+
+    consumer = asyncio.create_task(consume())
+    await asyncio.sleep(0.01)
+    consumer.cancel()
+    # The iterator swallows CancelledError internally and exits cleanly
+    await consumer
+
+
+@pytest.mark.asyncio
+async def test_transcription_completed_adds_new_user_item():
+    model = _DummyModel()
+    agent = RealtimeAgent(name="agent")
+    session = RealtimeSession(model, agent, None)
+
+    event = RealtimeModelInputAudioTranscriptionCompletedEvent(item_id="item1", transcript="hello")
+    await session.on_event(event)
+
+    # Should have appended a new user item
+    assert len(session._history) == 1
+    assert session._history[0].type == "message"
+    assert session._history[0].role == "user"
+
+
+class _FakeAudio:
+    # Looks like an audio part but is not an InputAudio/AssistantAudio instance
+    type = "audio"
+    transcript = None
+
+
+@pytest.mark.asyncio
+async def test_item_updated_merge_exception_path_logs_error(monkeypatch):
+    model = _DummyModel()
+    agent = RealtimeAgent(name="agent")
+    session = RealtimeSession(model, agent, None)
+
+    # existing assistant message with transcript to preserve
+    existing = AssistantMessageItem(
+        item_id="a1", role="assistant", content=[AssistantAudio(audio=None, transcript="t")]
+    )
+    session._history = [existing]
+
+    # incoming message with a deliberately bogus content entry to trigger assertion path
+    incoming = AssistantMessageItem(
+        item_id="a1", role="assistant", content=[AssistantAudio(audio=None, transcript=None)]
+    )
+    incoming.content[0] = cast(Any, _FakeAudio())
+
+    with patch("agents.realtime.session.logger") as mock_logger:
+        await session.on_event(RealtimeModelItemUpdatedEvent(item=incoming))
+        # error branch should be hit
+        assert mock_logger.error.called
+
+
+@pytest.mark.asyncio
+async def test_handle_tool_call_handoff_invalid_result_raises():
+    model = _DummyModel()
+    target = RealtimeAgent(name="target")
+
+    bad_handoff = Handoff(
+        tool_name="switch",
+        tool_description="",
+        input_json_schema={},
+        on_invoke_handoff=AsyncMock(return_value=123),  # invalid return
+        input_filter=None,
+        agent_name=target.name,
+        is_enabled=True,
+    )
+
+    agent = RealtimeAgent(name="agent", handoffs=[bad_handoff])
+    session = RealtimeSession(model, agent, None)
+
+    with pytest.raises(UserError):
+        await session._handle_tool_call(
+            RealtimeModelToolCallEvent(name="switch", call_id="c1", arguments="{}")
+        )
+
+
+@pytest.mark.asyncio
+async def test_on_guardrail_task_done_emits_error_event():
+    model = _DummyModel()
+    agent = RealtimeAgent(name="agent")
+    session = RealtimeSession(model, agent, None)
+
+    async def failing_task():
+        raise ValueError("task failed")
+
+    task = asyncio.create_task(failing_task())
+    # Wait for it to finish so exception() is available
+    try:
+        await task
+    except Exception:  # noqa: S110
+        pass
+
+    session._on_guardrail_task_done(task)
+
+    # Allow event task to enqueue
+    await asyncio.sleep(0.01)
+
+    # Should have a RealtimeError queued
+    err = await session._event_queue.get()
+    assert isinstance(err, RealtimeError)
+
+
+@pytest.mark.asyncio
+async def test_get_handoffs_async_is_enabled(monkeypatch):
+    # Agent includes both a direct Handoff and a RealtimeAgent (auto-converted)
+    target = RealtimeAgent(name="target")
+    other = RealtimeAgent(name="other")
+
+    async def is_enabled(ctx, agent):
+        return True
+
+    # direct handoff with async is_enabled
+    direct = Handoff(
+        tool_name="to_target",
+        tool_description="",
+        input_json_schema={},
+        on_invoke_handoff=AsyncMock(return_value=target),
+        input_filter=None,
+        agent_name=target.name,
+        is_enabled=is_enabled,
+    )
+
+    a = RealtimeAgent(name="a", handoffs=[direct, other])
+    session = RealtimeSession(_DummyModel(), a, None)
+
+    enabled = await RealtimeSession._get_handoffs(a, session._context_wrapper)
+    # Both should be enabled
+    assert len(enabled) == 2
+
+
 class MockRealtimeModel(RealtimeModel):
     def __init__(self):
         super().__init__()
@@ -104,6 +297,7 @@ def mock_agent():
     agent.get_all_tools = AsyncMock(return_value=[])
 
     type(agent).handoffs = PropertyMock(return_value=[])
+    type(agent).output_guardrails = PropertyMock(return_value=[])
     return agent
 
 
@@ -367,8 +561,13 @@ class TestEventHandling:
 
     @pytest.mark.asyncio
     async def test_function_call_event_triggers_tool_handling(self, mock_model, mock_agent):
-        """Test that function_call events trigger tool call handling"""
-        session = RealtimeSession(mock_model, mock_agent, None)
+        """Test that function_call events trigger tool call handling synchronously when disabled"""
+        session = RealtimeSession(
+            mock_model,
+            mock_agent,
+            None,
+            run_config={"async_tool_calls": False},
+        )
 
         # Create function call event
         function_call_event = RealtimeModelToolCallEvent(
@@ -384,7 +583,9 @@ class TestEventHandling:
             await session.on_event(function_call_event)
 
             # Should have called the tool handler
-            handle_tool_call_mock.assert_called_once_with(function_call_event)
+            handle_tool_call_mock.assert_called_once_with(
+                function_call_event, agent_snapshot=mock_agent
+            )
 
             # Should still have raw event
             assert session._event_queue.qsize() == 1
@@ -392,6 +593,36 @@ class TestEventHandling:
             assert isinstance(raw_event, RealtimeRawModelEvent)
             assert raw_event.data == function_call_event
 
+    @pytest.mark.asyncio
+    async def test_function_call_event_runs_async_by_default(self, mock_model, mock_agent):
+        """Function call handling should be scheduled asynchronously by default"""
+        session = RealtimeSession(mock_model, mock_agent, None)
+
+        function_call_event = RealtimeModelToolCallEvent(
+            name="test_function",
+            call_id="call_async",
+            arguments='{"param": "value"}',
+        )
+
+        with pytest.MonkeyPatch().context() as m:
+            handle_tool_call_mock = AsyncMock()
+            m.setattr(session, "_handle_tool_call", handle_tool_call_mock)
+
+            await session.on_event(function_call_event)
+
+            # Let the background task run
+            await asyncio.sleep(0)
+
+            handle_tool_call_mock.assert_awaited_once_with(
+                function_call_event, agent_snapshot=mock_agent
+            )
+
+        # Raw event still enqueued
+        assert session._event_queue.qsize() == 1
+        raw_event = await session._event_queue.get()
+        assert isinstance(raw_event, RealtimeRawModelEvent)
+        assert raw_event.data == function_call_event
+
 
 class TestHistoryManagement:
     """Test suite for history management and audio transcription in
@@ -755,6 +986,7 @@ class TestToolCallExecution:
         assert isinstance(tool_start_event, RealtimeToolStart)
         assert tool_start_event.tool == mock_function_tool
         assert tool_start_event.agent == mock_agent
+        assert tool_start_event.arguments == '{"param": "value"}'
 
         # Check tool end event
         tool_end_event = await session._event_queue.get()
@@ -762,6 +994,7 @@ class TestToolCallExecution:
         assert tool_end_event.tool == mock_function_tool
         assert tool_end_event.output == "function_result"
         assert tool_end_event.agent == mock_agent
+        assert tool_end_event.arguments == '{"param": "value"}'
 
     @pytest.mark.asyncio
     async def test_function_tool_with_multiple_tools_available(self, mock_model, mock_agent):
@@ -880,6 +1113,7 @@ class TestToolCallExecution:
         assert session._event_queue.qsize() == 1
         tool_start_event = await session._event_queue.get()
         assert isinstance(tool_start_event, RealtimeToolStart)
+        assert tool_start_event.arguments == "{}"
 
         # But no tool output should have been sent and no end event queued
         assert len(mock_model.sent_tool_outputs) == 0
@@ -902,10 +1136,20 @@ class TestToolCallExecution:
 
         await session._handle_tool_call(tool_call_event)
 
-        # Verify arguments were passed correctly
+        # Verify arguments were passed correctly to tool
         call_args = mock_function_tool.on_invoke_tool.call_args
         assert call_args[0][1] == complex_args
 
+        # Verify tool_start event includes arguments
+        tool_start_event = await session._event_queue.get()
+        assert isinstance(tool_start_event, RealtimeToolStart)
+        assert tool_start_event.arguments == complex_args
+
+        # Verify tool_end event includes arguments
+        tool_end_event = await session._event_queue.get()
+        assert isinstance(tool_end_event, RealtimeToolEnd)
+        assert tool_end_event.arguments == complex_args
+
     @pytest.mark.asyncio
     async def test_tool_call_with_custom_call_id(self, mock_model, mock_agent, mock_function_tool):
         """Test that tool context receives correct call_id"""
@@ -1049,7 +1293,6 @@ class TestGuardrailFunctionality:
         await self._wait_for_guardrail_tasks(session)
 
         # Should have triggered guardrail and interrupted
-        assert session._interrupted_by_guardrail is True
         assert mock_model.interrupts_called == 1
         assert len(mock_model.sent_messages) == 1
         assert "triggered_guardrail" in mock_model.sent_messages[0]
@@ -1063,6 +1306,37 @@ class TestGuardrailFunctionality:
         assert len(guardrail_events) == 1
         assert guardrail_events[0].message == "this is more than ten characters"
 
+    @pytest.mark.asyncio
+    async def test_agent_and_run_config_guardrails_not_run_twice(self, mock_model):
+        """Guardrails shared by agent and run config should execute once."""
+
+        call_count = 0
+
+        def guardrail_func(context, agent, output):
+            nonlocal call_count
+            call_count += 1
+            return GuardrailFunctionOutput(output_info={}, tripwire_triggered=False)
+
+        shared_guardrail = OutputGuardrail(
+            guardrail_function=guardrail_func, name="shared_guardrail"
+        )
+
+        agent = RealtimeAgent(name="agent", output_guardrails=[shared_guardrail])
+        run_config: RealtimeRunConfig = {
+            "output_guardrails": [shared_guardrail],
+            "guardrails_settings": {"debounce_text_length": 5},
+        }
+
+        session = RealtimeSession(mock_model, agent, None, run_config=run_config)
+
+        await session.on_event(
+            RealtimeModelTranscriptDeltaEvent(item_id="item_1", delta="hello", response_id="resp_1")
+        )
+
+        await self._wait_for_guardrail_tasks(session)
+
+        assert call_count == 1
+
     @pytest.mark.asyncio
     async def test_transcript_delta_multiple_thresholds_same_item(
         self, mock_model, mock_agent, triggered_guardrail
@@ -1155,14 +1429,12 @@ class TestGuardrailFunctionality:
         # Wait for async guardrail tasks to complete
         await self._wait_for_guardrail_tasks(session)
 
-        assert session._interrupted_by_guardrail is True
         assert len(session._item_transcripts) == 1
 
         # End turn
         await session.on_event(RealtimeModelTurnEndedEvent())
 
         # State should be cleared
-        assert session._interrupted_by_guardrail is False
         assert len(session._item_transcripts) == 0
         assert len(session._item_guardrail_run_counts) == 0
 
@@ -1210,6 +1482,92 @@ class TestGuardrailFunctionality:
         assert len(guardrail_events) == 1
         assert len(guardrail_events[0].guardrail_results) == 2
 
+    @pytest.mark.asyncio
+    async def test_agent_output_guardrails_triggered(self, mock_model, triggered_guardrail):
+        """Test that guardrails defined on the agent are executed."""
+        agent = RealtimeAgent(name="agent", output_guardrails=[triggered_guardrail])
+        run_config: RealtimeRunConfig = {
+            "guardrails_settings": {"debounce_text_length": 10},
+        }
+
+        session = RealtimeSession(mock_model, agent, None, run_config=run_config)
+
+        transcript_event = RealtimeModelTranscriptDeltaEvent(
+            item_id="item_1", delta="this is more than ten characters", response_id="resp_1"
+        )
+
+        await session.on_event(transcript_event)
+        await self._wait_for_guardrail_tasks(session)
+
+        assert mock_model.interrupts_called == 1
+        assert len(mock_model.sent_messages) == 1
+        assert "triggered_guardrail" in mock_model.sent_messages[0]
+
+        events = []
+        while not session._event_queue.empty():
+            events.append(await session._event_queue.get())
+
+        guardrail_events = [e for e in events if isinstance(e, RealtimeGuardrailTripped)]
+        assert len(guardrail_events) == 1
+        assert guardrail_events[0].message == "this is more than ten characters"
+
+    @pytest.mark.asyncio
+    async def test_concurrent_guardrail_tasks_interrupt_once_per_response(self, mock_model):
+        """Even if multiple guardrail tasks trigger concurrently for the same response_id,
+        only the first should interrupt and send a message."""
+        import asyncio
+
+        # Barrier to release both guardrail tasks at the same time
+        start_event = asyncio.Event()
+
+        async def async_trigger_guardrail(context, agent, output):
+            await start_event.wait()
+            return GuardrailFunctionOutput(
+                output_info={"reason": "concurrent"}, tripwire_triggered=True
+            )
+
+        concurrent_guardrail = OutputGuardrail(
+            guardrail_function=async_trigger_guardrail, name="concurrent_trigger"
+        )
+
+        run_config: RealtimeRunConfig = {
+            "output_guardrails": [concurrent_guardrail],
+            "guardrails_settings": {"debounce_text_length": 5},
+        }
+
+        # Use a minimal agent (guardrails from run_config)
+        agent = RealtimeAgent(name="agent")
+        session = RealtimeSession(mock_model, agent, None, run_config=run_config)
+
+        # Two deltas for same item and response to enqueue two guardrail tasks
+        await session.on_event(
+            RealtimeModelTranscriptDeltaEvent(
+                item_id="item_1", delta="12345", response_id="resp_same"
+            )
+        )
+        await session.on_event(
+            RealtimeModelTranscriptDeltaEvent(
+                item_id="item_1", delta="67890", response_id="resp_same"
+            )
+        )
+
+        # Wait until both tasks are enqueued
+        for _ in range(50):
+            if len(session._guardrail_tasks) >= 2:
+                break
+            await asyncio.sleep(0.01)
+
+        # Release both tasks concurrently
+        start_event.set()
+
+        # Wait for completion
+        if session._guardrail_tasks:
+            await asyncio.gather(*session._guardrail_tasks, return_exceptions=True)
+
+        # Only one interrupt and one message should be sent
+        assert mock_model.interrupts_called == 1
+        assert len(mock_model.sent_messages) == 1
+
 
 class TestModelSettingsIntegration:
     """Test suite for model settings integration in RealtimeSession."""
@@ -1490,6 +1848,47 @@ class TestModelSettingsPrecedence:
             assert model_settings["tool_choice"] == "required"
             assert model_settings["output_audio_format"] == "g711_ulaw"
 
+    @pytest.mark.asyncio
+    async def test_model_settings_preserve_initial_settings_on_updates(self):
+        """Initial model settings should persist when we recompute settings for updates."""
+
+        agent = RealtimeAgent(name="test_agent", instructions="test")
+        agent.handoffs = []
+        agent.get_system_prompt = AsyncMock(return_value="test_prompt")  # type: ignore
+        agent.get_all_tools = AsyncMock(return_value=[])  # type: ignore
+
+        mock_model = Mock(spec=RealtimeModel)
+
+        initial_settings: RealtimeSessionModelSettings = {
+            "voice": "initial_voice",
+            "output_audio_format": "pcm16",
+        }
+
+        session = RealtimeSession(
+            model=mock_model,
+            agent=agent,
+            context=None,
+            model_config={"initial_model_settings": initial_settings},
+            run_config={},
+        )
+
+        async def mock_get_handoffs(cls, agent, context_wrapper):
+            return []
+
+        with pytest.MonkeyPatch().context() as m:
+            m.setattr(
+                "agents.realtime.session.RealtimeSession._get_handoffs",
+                mock_get_handoffs,
+            )
+
+            model_settings = await session._get_updated_model_settings_from_agent(
+                starting_settings=None,
+                agent=agent,
+            )
+
+        assert model_settings["voice"] == "initial_voice"
+        assert model_settings["output_audio_format"] == "pcm16"
+
 
 class TestUpdateAgentFunctionality:
     """Tests for update agent functionality in RealtimeSession"""
@@ -1510,3 +1909,65 @@ class TestUpdateAgentFunctionality:
 
         # Check that the current agent and session settings are updated
         assert session._current_agent == second_agent
+
+
+class TestTranscriptPreservation:
+    """Tests ensuring assistant transcripts are preserved across updates."""
+
+    @pytest.mark.asyncio
+    async def test_assistant_transcript_preserved_on_item_update(self, mock_model, mock_agent):
+        session = RealtimeSession(mock_model, mock_agent, None)
+
+        # Initial assistant message with audio transcript present (e.g., from first turn)
+        initial_item = AssistantMessageItem(
+            item_id="assist_1",
+            role="assistant",
+            content=[AssistantAudio(audio=None, transcript="Hello there")],
+        )
+        session._history = [initial_item]
+
+        # Later, the platform retrieves/updates the same item but without transcript populated
+        updated_without_transcript = AssistantMessageItem(
+            item_id="assist_1",
+            role="assistant",
+            content=[AssistantAudio(audio=None, transcript=None)],
+        )
+
+        await session.on_event(RealtimeModelItemUpdatedEvent(item=updated_without_transcript))
+
+        # Transcript should be preserved from existing history
+        assert len(session._history) == 1
+        preserved_item = cast(AssistantMessageItem, session._history[0])
+        assert isinstance(preserved_item.content[0], AssistantAudio)
+        assert preserved_item.content[0].transcript == "Hello there"
+
+    @pytest.mark.asyncio
+    async def test_assistant_transcript_can_fallback_to_deltas(self, mock_model, mock_agent):
+        session = RealtimeSession(mock_model, mock_agent, None)
+
+        # Simulate transcript deltas accumulated for an assistant item during generation
+        await session.on_event(
+            RealtimeModelTranscriptDeltaEvent(
+                item_id="assist_2", delta="partial transcript", response_id="resp_2"
+            )
+        )
+
+        # Add initial assistant message without transcript
+        initial_item = AssistantMessageItem(
+            item_id="assist_2",
+            role="assistant",
+            content=[AssistantAudio(audio=None, transcript=None)],
+        )
+        await session.on_event(RealtimeModelItemUpdatedEvent(item=initial_item))
+
+        # Later update still lacks transcript; merge should fallback to accumulated deltas
+        update_again = AssistantMessageItem(
+            item_id="assist_2",
+            role="assistant",
+            content=[AssistantAudio(audio=None, transcript=None)],
+        )
+        await session.on_event(RealtimeModelItemUpdatedEvent(item=update_again))
+
+        preserved_item = cast(AssistantMessageItem, session._history[0])
+        assert isinstance(preserved_item.content[0], AssistantAudio)
+        assert preserved_item.content[0].transcript == "partial transcript"
