@@ -55,6 +55,7 @@ class FakeStreamingModel(Model):
         tracing: ModelTracing,
         *,
         previous_response_id: str | None,
+        conversation_id: str | None,
         prompt: Any | None,
     ) -> ModelResponse:
         raise NotImplementedError("Not implemented")
@@ -70,6 +71,7 @@ class FakeStreamingModel(Model):
         tracing: ModelTracing,
         *,
         previous_response_id: str | None,
+        conversation_id: str | None,
         prompt: Any | None,
     ) -> AsyncIterator[TResponseStreamEvent]:
         output = self.get_next_output()
@@ -137,15 +139,23 @@ async def test_single_agent_workflow(monkeypatch) -> None:
             },
             {
                 "id": "1",
-                "content": [{"annotations": [], "text": "a_message", "type": "output_text"}],
+                "content": [
+                    {"annotations": [], "logprobs": [], "text": "a_message", "type": "output_text"}
+                ],
                 "role": "assistant",
                 "status": "completed",
                 "type": "message",
             },
-            {"call_id": "2", "output": "tool_result", "type": "function_call_output"},
+            {
+                "call_id": "2",
+                "output": "tool_result",
+                "type": "function_call_output",
+            },
             {
                 "id": "1",
-                "content": [{"annotations": [], "text": "done", "type": "output_text"}],
+                "content": [
+                    {"annotations": [], "logprobs": [], "text": "done", "type": "output_text"}
+                ],
                 "role": "assistant",
                 "status": "completed",
                 "type": "message",
@@ -173,15 +183,23 @@ async def test_single_agent_workflow(monkeypatch) -> None:
             },
             {
                 "id": "1",
-                "content": [{"annotations": [], "text": "a_message", "type": "output_text"}],
+                "content": [
+                    {"annotations": [], "logprobs": [], "text": "a_message", "type": "output_text"}
+                ],
                 "role": "assistant",
                 "status": "completed",
                 "type": "message",
             },
-            {"call_id": "2", "output": "tool_result", "type": "function_call_output"},
+            {
+                "call_id": "2",
+                "output": "tool_result",
+                "type": "function_call_output",
+            },
             {
                 "id": "1",
-                "content": [{"annotations": [], "text": "done", "type": "output_text"}],
+                "content": [
+                    {"annotations": [], "logprobs": [], "text": "done", "type": "output_text"}
+                ],
                 "role": "assistant",
                 "status": "completed",
                 "type": "message",
@@ -189,7 +207,9 @@ async def test_single_agent_workflow(monkeypatch) -> None:
             {"role": "user", "content": "transcription_2"},
             {
                 "id": "1",
-                "content": [{"annotations": [], "text": "done_2", "type": "output_text"}],
+                "content": [
+                    {"annotations": [], "logprobs": [], "text": "done_2", "type": "output_text"}
+                ],
                 "role": "assistant",
                 "status": "completed",
                 "type": "message",
